import os
import sys

TASK_CONFIG = {
    "enabled": True,
    "quiet": False,
    "broker_url": f'redis://{os.getenv("REDIS_HOST", "localhost")}:{os.getenv("REDIS_PORT", "6379")}/{os.getenv("REDIS_DB", "1")}',
    "beat_scheduler": "django_celery_beat.schedulers:DatabaseScheduler",
    "result_backend": "django-db",
    "task_track_started": True,
    "broker_connection_retry_on_startup": True,
    "worker_redirect_stdouts": False,
}

if "test" in sys.argv or any(["pytest" in arg for arg in sys.argv]):
    TASK_CONFIG["task_always_eager"] = True
    TASK_CONFIG["task_store_eager_result"] = True
    TASK_CONFIG["beat_scheduler"] = "celery.beat:PersistentScheduler"



from jaseci.svc import RedisService as Rs
from .config import REDIS_CONFIG


class RedisService(Rs):
    def build_config(self, hook) -> dict:
        return hook.service_glob("REDIS_CONFIG", REDIS_CONFIG)



from django.test import TestCase
from django.urls import NoReverseMatch
from jaseci.utils.utils import TestCaseHelper
from rest_framework.test import APIClient
from rest_framework import status

from django.contrib.auth import get_user_model
from jaseci_serv.base.models import GlobalVars
from django.urls import reverse


class ApiTest(TestCaseHelper, TestCase):
    def setUp(self):
        """Creates test client, admin user and user for this test case"""
        super().setUp()
        self.client = APIClient()
        self.admin_user = get_user_model().objects.create_superuser(
            email="JSCITEST_admin@jaseci.com", password="password"
        )
        self.user = get_user_model().objects.create_user(
            email="JSCITEST_test@jaseci.com", password="password"
        )
        self.client.force_authenticate(self.admin_user)
        self.master = self.user.get_master()

    def tearDown(self):
        """Deletes test users and sample stripe_api_key out of databases"""
        self.admin_user.delete()
        self.user.delete()
        super().tearDown()

    def test_stripe_init_returns_stripe_webhook_url(self):
        """/stripe/init"""
        # self.master._h.save_glob(
        #     "STRIPE_API_KEY",
        #     "sk_test_4eC39HqLyjWDarjtT1zdp7dc",
        # )
        GlobalVars.objects.create(
            name="STRIPE_API_KEY",
            value="sk_test_4eC39HqLyjWDarjtT1zdp7dc",
        )

        res = self.client.post(reverse("stripe_init"))

        # print(res.json())

        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["success"])
        self.assertTrue(res.json()["data"].startswith("/js_public/walker_callback/"))

    def test_stripe_init_should_return_forbidden_response(self):
        """should return forbidden response"""

        # self.master._h.destroy_glob(
        #     "STRIPE_API_KEY"
        # )

        GlobalVars.objects.filter(name="STRIPE_API_KEY").delete()

        if self.master._h.get_glob("STRIPE_API_KEY"):
            self.master._h.destroy_glob("STRIPE_API_KEY")

        res = self.client.post(reverse("stripe_init"))

        # print(res.json())

        self.assertEqual(res.status_code, status.HTTP_403_FORBIDDEN)
        self.assertFalse(res.json()["success"])
        self.assertTrue(
            res.json()["message"]
            == "Stripe is not yet configured. Please set a valid stripe key."
        )



from .models import StripeVars
from django.db import IntegrityError


class stripe_hook:
    stripe_key = None

    def get_key():
        """get stripe config by name"""

        if not __class__.stripe_key:
            key = StripeVars.objects.get(name="STRIPE_API_KEY").value
            __class__.stripe_key = key

        return __class__.stripe_key

    def set_key(value: str):
        """set stripe config by name"""

        __class__.save_obj("STRIPE_KEY", value)
        __class__.stripe_key = value

    def get_obj(name: str):
        """get stripe config by name"""
        try:
            return StripeVars.objects.get(name=name).value
        except StripeVars.DoesNotExist:
            print(f"{name} doesn't exist")
            return None

    def save_obj(name: str, value: str):
        """set stripe config"""
        try:
            data = {"value": value}

            obj = StripeVars.objects.update_or_create(name=name, defaults=data)

            return obj
        except IntegrityError as e:
            print(f"{name} is already exist")
            return None

    def list_obj():
        """get list of stripe config"""

        cfg_list = list(StripeVars.objects.all().values())

        # exclude id from the list
        cfg_list = [
            {key: val for key, val in sub.items() if key != "id"} for sub in cfg_list
        ]

        return cfg_list

    def delete_obj(name: str):
        """get stripe config by name"""
        try:
            obj = StripeVars.objects.get(name=name)
            obj.delete()

            return True
        except StripeVars.DoesNotExist:
            print(f"{name} doesn't exist")
            return None



from django.contrib import admin
from .models import StripeVars


class StripeVarsAdmin(admin.ModelAdmin):
    ordering = ["name"]
    list_display = ("name", "value")
    search_fields = ["name", "value"]


admin.site.register(StripeVars, StripeVarsAdmin)



from jaseci.svc import PrometheusService as Ps
from .config import PROMON_CONFIG


class PromotheusService(Ps):
    def build_config(self, hook) -> dict:
        return hook.service_glob("PROMON_CONFIG", PROMON_CONFIG)



from jaseci.api.interface import Interface
from jaseci.api.jsorc_api import JsOrcApi as CoreJsOrcApi
from .jsorc_loadtest import JsorcLoadTest


class JsOrcApi(CoreJsOrcApi):
    @Interface.admin_api()
    def jsorc_loadtest(self, test: str, experiment: str = "", mem: int = 0):
        """
        A jsorc loadtest
        """
        tester = JsorcLoadTest(test)
        return tester.run_test(experiment, mem)



from jaseci.svc import TaskService as Ts
from .config import TASK_CONFIG


#################################################
#                 TASK APP ORM                  #
#################################################


class TaskService(Ts):
    def build_config(self, hook) -> dict:
        return hook.service_glob("TASK_CONFIG", TASK_CONFIG)



import uuid
import os.path

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from rest_framework.permissions import IsAdminUser, IsAuthenticated
from .models import JSX_STRIPE_DIR
from jaseci_serv.base.models import JaseciObject


class StripeView(APIView):

    http_method_names = ["post"]
    permission_classes = (IsAuthenticated, IsAdminUser)

    def post(self, request):
        master = request.user.get_master()
        api_key = master._h.get_glob("STRIPE_API_KEY")

        if not api_key:
            return Response(
                {
                    "message": "Stripe is not yet configured. Please set a valid stripe key.",
                    "success": False,
                },
                status=status.HTTP_403_FORBIDDEN,
            )

        try:
            STRIPE_JAC = os.path.join(JSX_STRIPE_DIR, "stripe.jac")
            FILE = open(STRIPE_JAC, "r")
            code = FILE.read()

            # register stripe.jac
            stripe_sentinel = master.sentinel_register(
                name="stripe_sentinel", code=code
            )

            master.save()
            master._h.commit()

            stripe_sentinel = master._h.get_obj(
                master._m_id, str(uuid.UUID(stripe_sentinel[0]["jid"]))
            )

            stripe_webhook_arch = stripe_sentinel.arch_ids.get_obj_by_name(
                "stripe_webhook"
            )

            stripe_webhook_wlk = None
            wlk = JaseciObject.objects.filter(
                name="stripe_webhook", j_type="walker"
            ).exists()

            if stripe_webhook_arch and not wlk:
                # if theres stripe_webhook architype and not yet have stripe_webhook walker
                # run run_architype to create walker
                stripe_webhook_wlk = stripe_webhook_arch.run_architype(
                    jac_ast=stripe_webhook_arch.get_jac_ast()
                )
            else:
                stripe_webhook_wlk = JaseciObject.objects.get(
                    name="stripe_webhook", j_type="walker"
                )

                stripe_webhook_wlk = master._h.get_obj(
                    master._m_id, stripe_webhook_wlk.jid
                )

            # if theres NO stripe_webhook architype and no stripe_webhook walker, return an error response
            if not stripe_webhook_arch and not stripe_webhook_wlk:
                return Response(
                    {
                        "message": "Walker stripe_webhook not found!",
                        "success": False,
                    },
                    status=status.HTTP_403_FORBIDDEN,
                )

            keys = stripe_webhook_wlk.namespace_keys()["anyone"]

            master.save()
            master._h.commit()

            root_node = master.graph_active_get()["jid"].split(":")[2]

            stripe_webhook_url = f"/js_public/walker_callback/{root_node}/{str(stripe_webhook_wlk.id)}?key={keys}"
            FILE.close()

            return Response({"data": stripe_webhook_url, "success": True})
        except Exception as e:
            return Response({"message": str(e), "success": False})



"""
Django settings for jaseci project.

Generated by 'django-admin startproject' using Django 3.0.3.

For more information on this file, see
https://docs.djangoproject.com/en/3.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.0/ref/settings/
"""

import os
import sys

from .jsx_oauth.config import *

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

TEMPLATE_DIR = os.path.join(BASE_DIR, "ui/templates")

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "@%ad_kyis62uf7qf^w9kv(8$db4)%c$nnnjk^us=s@-gj*)aal"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]

# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "rest_framework",
    "knox",
    "django_rest_passwordreset",
    "drf_yasg",
    "jaseci_serv.base",
    "jaseci_serv.user_api",
    "jaseci_serv.obj_api",
    "jaseci_serv.jac_api",
    "corsheaders",
    "django_celery_results",
    "django_celery_beat",
    "jaseci_serv.jsx_stripe",
    "jaseci_serv.jsx_oauth",
]

MIDDLEWARE = [
    "corsheaders.middleware.CorsMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

CORS_ORIGIN_ALLOW_ALL = True

ROOT_URLCONF = "jaseci_serv.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [TEMPLATE_DIR],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "jaseci_serv.wsgi.application"

# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

if "RDS_HOSTNAME" in os.environ:
    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.mysql",
            "NAME": os.environ["RDS_DB_NAME"],
            "USER": os.environ["RDS_USERNAME"],
            "PASSWORD": os.environ["RDS_PASSWORD"],
            "HOST": os.environ["RDS_HOSTNAME"],
            "PORT": os.environ["RDS_PORT"],
        }
    }

elif "POSTGRES_HOST" in os.environ:
    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.postgresql",
            "HOST": os.environ.get("POSTGRES_HOST"),
            "NAME": os.environ.get("POSTGRES_USER"),
            "USER": os.environ.get("POSTGRES_USER"),
            "PASSWORD": os.environ.get("POSTGRES_PASSWORD"),
            "PORT": os.getenv("POSTGRES_PORT", 5432),
        }
    }

elif "test" in sys.argv or any(["pytest" in arg for arg in sys.argv]):
    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.sqlite3",
            "NAME": ":memory:",
        }
    }
else:
    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.sqlite3",
            "NAME": "mydatabase",
            "TEST": {"NAME": "test"},
        }
    }

# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation."
        + "UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation." + "MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation." + "CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation." + "NumericPasswordValidator",
    },
]

# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = False

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

STATIC_URL = "/static/"
STATICFILES_DIRS = [os.path.join(BASE_DIR, "static")]


# Sets global default user model to be custom
AUTH_USER_MODEL = "base.User"

REST_FRAMEWORK = {
    "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.PageNumberPagination",
    "PAGE_SIZE": 100,
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "knox.auth.TokenAuthentication",
    ],
}

REST_KNOX = {
    "TOKEN_TTL": None,
}

LOGIN_URL = "/ui/login"
LOGIN_REDIRECT_URL = "/ui"
LOGOUT_REDIRECT_URL = "/ui"

# Configs to Manage by Jaseci
JASECI_CONFIGS = [
    "EMAIL_BACKEND",
    "EMAIL_USE_TLS",
    "EMAIL_HOST",
    "EMAIL_HOST_USER",
    "EMAIL_HOST_PASSWORD",
    "EMAIL_PORT",
    "EMAIL_DEFAULT_FROM",
    "EMAIL_ACTIVATION_SUBJ",
    "EMAIL_ACTIVATION_BODY",
    "EMAIL_ACTIVATION_HTML_BODY",
    "EMAIL_RESETPASS_SUBJ",
    "EMAIL_RESETPASS_BODY",
    "EMAIL_RESETPASS_HTML_BODY",
    "LOGIN_TOKEN_TTL_HOURS",
]

DJANGO_CELERY_BEAT_TZ_AWARE = False
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

JSX_STRIPE_DIR = os.path.join(BASE_DIR, "jaseci_serv/jsx_stripe")


#################################################
#                    ADDONS                     #
#################################################

# ------------------- OAUTH ------------------- #

INSTALLED_APPS += OAUTH_APPS
JSX_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
TEMPLATES[0]["DIRS"] += [os.path.join(JSX_DIR, "templates")]



import random
import string
from allauth.socialaccount.providers.google.views import GoogleOAuth2Adapter
from allauth.socialaccount.providers.facebook.views import FacebookOAuth2Adapter
from allauth.socialaccount.providers.github.views import GitHubOAuth2Adapter
from allauth.socialaccount.providers.microsoft.views import MicrosoftGraphOAuth2Adapter
from allauth.socialaccount.providers.okta.views import OktaOAuth2Adapter
from allauth.socialaccount.providers.oauth2.client import OAuth2Client
from rest_framework.generics import GenericAPIView
from rest_framework.response import Response
from jaseci_serv.jsx_oauth import serializers
from django.conf import settings
from django.views import View
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _
from rest_framework import status
from rest_framework.generics import GenericAPIView
from rest_framework.response import Response
from jaseci_serv.jsx_oauth.models import SocialLoginProvider, PROVIDERS_MAPPING
from allauth.socialaccount.models import SocialApp
from jaseci_serv.jsx_oauth.utils import JSXSocialLoginView


class ExampleUrlView(GenericAPIView):
    serializer_class = serializers.GetExampleUrlSerializer

    def post(self, request):
        serializer_data = self.serializer_class(data=request.data)
        serializer_data.is_valid(raise_exception=True)
        provider = serializer_data.validated_data.get("provider")
        callback_url = f"{request.build_absolute_uri('/')[:-1]}{settings.DEFAULT_CALLBACK_URL_FOR_SSO}"
        social_app = SocialApp.objects.filter(provider=provider.lower()).first()
        if social_app:
            return Response(
                {
                    "url": PROVIDERS_MAPPING[provider]["LOGIN_URL"].format(
                        callback_url=callback_url, client_id=social_app.client_id
                    )
                }
            )
        else:
            return Response(
                {
                    "error": f"Social login applications for the {provider} not configured properly"
                },
                status=status.HTTP_400_BAD_REQUEST,
            )


class GoogleLogin(JSXSocialLoginView):
    adapter_class = GoogleOAuth2Adapter
    provider = SocialLoginProvider.GOOGLE
    client_class = OAuth2Client


class FaceBookLogin(JSXSocialLoginView):
    adapter_class = FacebookOAuth2Adapter
    provider = SocialLoginProvider.FACEBOOK
    client_class = OAuth2Client


class GitHubLogin(JSXSocialLoginView):
    adapter_class = GitHubOAuth2Adapter
    provider = SocialLoginProvider.GITHUB
    client_class = OAuth2Client


class OktaLogin(JSXSocialLoginView):
    adapter_class = OktaOAuth2Adapter
    provider = SocialLoginProvider.OKTA
    client_class = OAuth2Client


class MicrosoftLogin(JSXSocialLoginView):
    adapter_class = MicrosoftGraphOAuth2Adapter
    provider = SocialLoginProvider.MICROSOFT
    client_class = OAuth2Client


class OpenIdLogin(JSXSocialLoginView):
    adapter_class = GitHubOAuth2Adapter  # TODO: create custom adaptor for OpenID
    provider = SocialLoginProvider.OPENID
    client_class = OAuth2Client


class ExampleLiveView(View):
    template_name = "examples/social_auth.html"

    def get(self, request, provider):
        code = request.GET.get("code")
        social_app = SocialApp.objects.filter(provider=provider).first()
        return render(
            request,
            self.template_name,
            {
                "provider": provider,
                "code": code,
                "client_id": social_app.client_id,
                "state": self.generate_state(),
            },
        )

    def generate_state(self):
        return "".join(
            random.choices(
                string.ascii_uppercase + string.ascii_lowercase + string.digits, k=64
            )
        )



from django.db import models
import os.path
from ..settings import JSX_STRIPE_DIR


STRIPE_EVENTS_FILE = os.path.join(JSX_STRIPE_DIR, "stripe_events.txt")

COLOR_CHOICES = ()
STRIPE_EVENTS = []

if not os.path.isfile(STRIPE_EVENTS_FILE):
    print("File does not exist.")
else:
    f = open(STRIPE_EVENTS_FILE, "r")
    lines = f.readlines()

    for line in lines:
        STRIPE_EVENTS.append(line.replace("\n", ""))
        COLOR_CHOICES = COLOR_CHOICES + (
            (line.replace("\n", ""), line.replace("\n", "").replace(".", "_").upper()),
        )


class StripeVars(models.Model):
    """Stripe configuration item"""

    name = models.CharField(max_length=31, unique=True, choices=COLOR_CHOICES)
    value = models.TextField(blank=True)



from jaseci.svc import MetaService as Ms
from jaseci_serv.svc import MailService, RedisService, TaskService, PromotheusService
from .config import RUN_SVCS


class MetaService(Ms):

    ###################################################
    #                   OVERRIDEN                     #
    ###################################################
    def __init__(self):
        super().__init__(run_svcs=RUN_SVCS)

    ###################################################
    #                   OVERRIDEN                     #
    ###################################################

    def populate_context(self):
        from jaseci_serv.hook.orm import OrmHook
        from jaseci_serv.base.models import (
            Master,
            SuperMaster,
            GlobalVars,
            JaseciObject,
        )

        self.add_context(
            "hook", OrmHook, objects=JaseciObject.objects, globs=GlobalVars.objects
        )
        self.add_context("master", Master)
        self.add_context("super_master", SuperMaster)

    def populate_services(self):
        self.add_service_builder("redis", RedisService)
        self.add_service_builder("task", TaskService)
        self.add_service_builder("mail", MailService)
        self.add_service_builder("promon", PromotheusService)



from jaseci_serv.svc import MetaService
from jaseci_serv.jsx_oauth.models import PROVIDERS_MAPPING
from allauth.socialaccount.providers.oauth2.client import OAuth2Client
from dj_rest_auth.registration.views import SocialLoginView
from dj_rest_auth.registration.serializers import SocialLoginSerializer
from jaseci_serv.base.models import lookup_global_config
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from knox.settings import knox_settings
from knox.models import AuthToken
from django.contrib.auth import authenticate, get_user_model
from datetime import timedelta
from rest_framework.response import Response
from rest_framework.exceptions import APIException
from rest_framework import serializers
from allauth.socialaccount.helpers import complete_social_login
from allauth.account import app_settings as allauth_settings
from requests.exceptions import HTTPError


class RegistrationConflict(APIException):
    status_code = 409
    default_detail = "User is already registered with this e-mail address!"
    default_code = "Registration failed!"


class JSXSocialLoginSerializer(SocialLoginSerializer):
    def validate(self, attrs):
        view = self.context.get("view")
        request = self._get_request()

        if not view:
            raise serializers.ValidationError(
                _("View is not defined, pass it as a context variable"),
            )

        adapter_class = getattr(view, "adapter_class", None)
        if not adapter_class:
            raise serializers.ValidationError(_("Define adapter_class in view"))

        adapter = adapter_class(request)
        app = adapter.get_provider().get_app(request)

        access_token = attrs.get("access_token")
        code = attrs.get("code")
        # Case 1: We received the access_token
        if access_token:
            tokens_to_parse = {"access_token": access_token}
            token = access_token
            # For sign in with apple
            id_token = attrs.get("id_token")
            if id_token:
                tokens_to_parse["id_token"] = id_token

        # Case 2: We received the authorization code
        elif code:
            self.set_callback_url(view=view, adapter_class=adapter_class)
            self.client_class = getattr(view, "client_class", None)

            if not self.client_class:
                raise serializers.ValidationError(
                    _("Define client_class in view"),
                )
            provider = adapter.get_provider()
            scope = provider.get_scope(request)
            client = self.client_class(
                request,
                app.client_id,
                app.secret,
                adapter.access_token_method,
                adapter.access_token_url,
                self.callback_url,
                scope,
                scope_delimiter=adapter.scope_delimiter,
                headers=adapter.headers,
                basic_auth=adapter.basic_auth,
            )
            token = client.get_access_token(code)
            access_token = token["access_token"]
            tokens_to_parse = {"access_token": access_token}

            # If available we add additional data to the dictionary
            for key in ["refresh_token", "id_token", adapter.expires_in_key]:
                if key in token:
                    tokens_to_parse[key] = token[key]
        else:
            raise serializers.ValidationError(
                _("Incorrect input. access_token or code is required."),
            )

        social_token = adapter.parse_token(tokens_to_parse)
        social_token.app = app

        try:
            login = self.get_social_login(adapter, app, social_token, token)
            complete_social_login(request, login)
        except HTTPError:
            raise serializers.ValidationError(_("Incorrect value"))
        if not login.is_existing:
            if allauth_settings.UNIQUE_EMAIL:
                # Do we have an account already with this email address?
                account_exists = (
                    get_user_model()
                    .objects.filter(
                        email=login.user.email,
                    )
                    .exists()
                )
                if account_exists:
                    raise RegistrationConflict()

            login.lookup()
            login.save(request, connect=True)
        attrs["user"] = login.account.user
        if "name" in login.account.extra_data.keys():
            attrs["name"] = login.account.extra_data.get("name", "")
        return attrs


class JSXSocialLoginView(SocialLoginView):
    adapter_class = None
    provider = None
    client_class = OAuth2Client
    serializer_class = JSXSocialLoginSerializer
    # permission_classes = [IsValidateLicense, AllowAny]

    def get_callback_url(self, request):
        callback_url = request.POST.get("callback_url")
        if callback_url:
            return callback_url

        if self.provider:
            prov = PROVIDERS_MAPPING[self.provider]
            return lookup_global_config(
                name=prov["URL_KEY"],
                default=f'{request.build_absolute_uri("/")[:-1]}{prov["DEFAULT_REDIRECT_URI"]}',
            )
        raise RuntimeError(
            "Provider name cannot be empty or None. "
            'Please provide a valid provider name e.g. "GOOGLE"'
        )

    def dispatch(self, request, *args, **kwargs):
        self.callback_url = self.get_callback_url(request)
        return super().dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.request = request
        self.serializer = self.get_serializer(data=self.request.data)
        self.serializer.is_valid(raise_exception=True)
        self.user = self.serializer.validated_data.get("user")
        # self.login()
        self.user.is_activated = True
        self.user.name = self.serializer.validated_data.get("name")
        self.user.master = (
            MetaService().build_master(h=self.user._h, name=self.user.email).id
        )
        self.user._h.commit()
        self.user.save()
        auth_token = AuthToken.objects.filter(user_id=self.user.id)
        if auth_token:
            AuthToken.objects.filter(user_id=self.user.id).delete()
        instance, token = AuthToken.objects.create(
            self.user,
            knox_settings.TOKEN_TTL
            if knox_settings.TOKEN_TTL
            else timedelta(hours=settings.KNOX_TOKEN_EXPIRY),
        )

        auth_user = authenticate(
            request=self.request, username=self.user.email, password=self.user.password
        )
        return Response(
            {
                "email": auth_user.email if auth_user else self.user.email,
                "token": token,
                "exp": instance.expiry,
            }
        )



from django.contrib.auth import get_user_model
from django.urls import reverse

from rest_framework.test import APIClient

from time import sleep


class JsorcLoadTest:
    """
    A load tester module around JSORC.
    TODO: add experiment test cases
    """

    def __init__(self, test):
        self.client = APIClient()
        user_email = "JSCITfdfdEST_test@jaseci.com"
        suser_email = "JSCITfdfdEST_test2@jaseci.com"
        password = "password"
        try:
            self.user = get_user_model().objects.get(email=user_email.lower())
        except get_user_model().DoesNotExist:
            self.user = get_user_model().objects.create_user(user_email, password)
        try:
            self.suser = get_user_model().objects.get(email=suser_email.lower())
        except get_user_model().DoesNotExist:
            self.suser = get_user_model().objects.create_superuser(
                suser_email, password
            )

        self.auth_client = APIClient()
        self.auth_client.force_authenticate(self.user)
        self.sauth_client = APIClient()
        self.sauth_client.force_authenticate(self.suser)

        self.test = test

    def run_test(self, experiment, mem):
        """
        Run the corresponding jsorc test
        """
        test_func = getattr(self, self.test)
        if experiment == "":
            return test_func()
        else:
            return test_func(experiment, mem)

    def load_action(self, name, mode, wait_for_ready=False):
        """
        Load action in corresponding mode.
        If wait is True, this function will block until the action is fully loaded, including microserivice is up and connected etc.
        """
        payload = {"op": "jsorc_actions_load", "name": name, "mode": mode}
        res = self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        while wait_for_ready:
            mode = "module" if mode == "local" else mode
            if res.data["action_status"]["mode"] == mode:
                break
            sleep(10)
            payload = {"op": "jsorc_actions_load", "name": name, "mode": mode}
            res = self.sauth_client.post(
                reverse(f'jac_api:{payload["op"]}'), payload, format="json"
            )

    def unload_action(self, name, mode, retire_svc=False):
        payload = {
            "op": "jsorc_actions_unload",
            "name": name,
            "mode": mode,
            "retire_svc": retire_svc,
        }
        res = self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        return res.data

    def run_walker(self, walker_name, ctx={}):
        payload = {"op": "walker_run", "name": walker_name, "ctx": ctx}
        res = self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        return res.data

    def sentinel_register(self, jac_file):
        jac_code = open(jac_file).read()
        # TODO: remove this once opt_level bug is fixed
        payload = {"op": "sentinel_register", "code": jac_code, "opt_level": 2}
        res = self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )

    def set_jsorc_actionpolicy(self, policy_name, policy_params):
        payload = {
            "op": "jsorc_actionpolicy_set",
            "policy_name": policy_name,
            "policy_params": policy_params,
        }
        res = self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )

    def start_benchmark(self):
        # Start benchmark
        payload = {"op": "jsorc_benchmark_start"}
        self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )

    def stop_benchmark(self):
        # Stop benchmark and get report
        payload = {"op": "jsorc_benchmark_stop", "report": True}
        res = self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        return res.data

    def start_actions_tracking(self):
        payload = {"op": "jsorc_actionstracking_start"}
        res = self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        return res.data

    def stop_actions_tracking(self):
        payload = {"op": "jsorc_actionstracking_stop"}
        res = self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        return res.data



import uuid
import base64
import os

from django.test import TestCase
from django.urls import reverse
from ..models import JSX_STRIPE_DIR
from jaseci.utils.utils import TestCaseHelper
from rest_framework.test import APIClient
from rest_framework import status


from django.contrib.auth import get_user_model
from jaseci_serv.base.models import GlobalVars


class testStripeJac(TestCaseHelper, TestCase):
    def setUp(self):
        """Creates test client, admin user and user for this test case"""
        super().setUp()
        self.client = APIClient()
        self.admin_user = get_user_model().objects.create_superuser(
            email="JSCITEST_admin@jaseci.com", password="password"
        )
        self.user = get_user_model().objects.create_user(
            email="JSCITEST_test@jaseci.com", password="password"
        )
        self.client.force_authenticate(self.admin_user)
        self.master = self.user.get_master()

        STRIPE_JAC = os.path.join(JSX_STRIPE_DIR, "stripe.jac")
        ll_file = base64.b64encode(open(STRIPE_JAC).read().encode()).decode()
        payload = {
            "op": "sentinel_register",
            "name": "Something",
            "code": ll_file,
            "encoded": True,
        }

        res = self.client.post(reverse("jac_api:sentinel_register"), payload)
        self.snt = self.master._h.get_obj(
            self.master.jid, str(uuid.UUID(res.data[0]["jid"]))
        )
        self.gph = self.master._h.get_obj(
            self.master.jid, str(uuid.UUID(res.data[1]["jid"]))
        )

        GlobalVars.objects.create(
            name="STRIPE_API_KEY",
            value="sk_test_4eC39HqLyjWDarjtT1zdp7dc",
        )

        res = self.client.post(reverse("stripe_init"))

        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["success"])
        self.assertTrue(res.json()["data"].startswith("/js_public/walker_callback/"))

    def test_create_product(self):
        """test create_product walker"""

        payload = {
            "name": "create_product",
            "ctx": {
                "name": "test@gmail.com",
                "description": "test user",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_create_product_price(self):
        """test create_product_price walker"""

        payload = {
            "name": "create_product_price",
            "ctx": {
                "productId": "testProductId",
                "amount": 12,
                "currency": "usd",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_product_list(self):
        """test product_list walker"""

        payload = {
            "name": "product_list",
            "ctx": {
                "detailed": True,
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_create_product_price(self):
        """test create_product_price walker"""

        payload = {
            "name": "create_product_price",
            "ctx": {
                "productId": "testProductId",
                "amount": 12,
                "currency": "usd",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_create_customer(self):
        """test create_customer walker"""

        payload = {
            "name": "create_customer",
            "ctx": {
                "email": "test@gmail.com",
                "name": "test user",
                "metadata": {"user_id": "123", "node_id": "test_node_id"},
                "payment_method_id": "test_payment_method",
                "address": "my address",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_get_customer(self):
        """test get_customer walker"""

        payload = {
            "name": "get_customer",
            "ctx": {
                "customer_id": "customer_id1",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_attach_payment_method(self):
        """test attach_payment_method walker"""

        payload = {
            "name": "attach_payment_method",
            "ctx": {
                "payment_method_id": "payment_method_id",
                "customer_id": "customer_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_delete_payment_method(self):
        """test delete_payment_method walker"""

        payload = {
            "name": "delete_payment_method",
            "ctx": {
                "payment_method_id": "payment_method_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_get_payment_methods(self):
        """test get_payment_methods walker"""

        payload = {
            "name": "get_payment_methods",
            "ctx": {
                "customer_id": "customer_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_update_default_payment_method(self):
        """test update_default_payment_method walker"""

        payload = {
            "name": "update_default_payment_method",
            "ctx": {
                "payment_method_id": "payment_method_id",
                "customer_id": "customer_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_create_invoice(self):
        """test create_invoice walker"""

        payload = {
            "name": "create_invoice",
            "ctx": {
                "customer_id": "customer_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_get_invoice_list(self):
        """test get_invoice_list walker"""

        payload = {
            "name": "get_invoice_list",
            "ctx": {
                "customer_id": "customer_id",
                "subscription_id": "subscription_id",
                "starting_after": "starting_after",
                "limit": 12,
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_get_payment_intents(self):
        """test get_payment_intents walker"""

        payload = {
            "name": "get_payment_intents",
            "ctx": {
                "customer_id": "customer_id",
                "starting_after": "starting_after",
                "limit": 12,
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_create_payment_intents(self):
        """test create_payment_intents walker"""

        payload = {
            "name": "create_payment_intents",
            "ctx": {
                "amount": 12,
                "currency": "currency",
                "payment_method_types": "payment_method_types",
                "customer_id": "customer_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_get_customer_subscription(self):
        """test get_customer_subscription walker"""

        payload = {
            "name": "get_customer_subscription",
            "ctx": {
                "customer_id": "customer_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_create_payment_method(self):
        """test create_payment_method walker"""

        payload = {
            "name": "create_payment_method",
            "ctx": {
                "card_type": "card_type",
                "card": "card",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_create_trial_subscription(self):
        """test create_trial_subscription walker"""

        payload = {
            "name": "create_trial_subscription",
            "ctx": {
                "payment_method_id": "card_type",
                "price_id": "card",
                "customer_id": "card",
                "trial_period_days": 30,
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_create_subscription(self):
        """test create_subscription walker"""

        payload = {
            "name": "create_subscription",
            "ctx": {
                "payment_method_id": "card_type",
                "price_id": "card",
                "customer_id": "card",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_cancel_subscription(self):
        """test cancel_subscription walker"""

        payload = {
            "name": "cancel_subscription",
            "ctx": {
                "subscription_id": "subscription_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_get_subscription(self):
        """test get_subscription walker"""

        payload = {
            "name": "get_subscription",
            "ctx": {
                "subscription_id": "subscription_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_update_subscription(self):
        """test update_subscription walker"""

        payload = {
            "name": "update_subscription",
            "ctx": {
                "subscription_id": "subscription_id",
                "subscription_item_id": "subscription_item_id",
                "price_id": "price_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_get_invoice(self):
        """test get_invoice walker"""

        payload = {
            "name": "get_invoice",
            "ctx": {
                "invoice_id": "invoice_id",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])

    def test_create_usage_report(self):
        """test create_usage_report walker"""

        payload = {
            "name": "create_usage_report",
            "ctx": {
                "subscription_item_id": "subscription_item_id",
                "quantity": "quantity",
                "mock_api": True,
            },
        }

        res = self.client.post("/js/walker_run", payload, format="json")
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertTrue(res.json()["report"][0]["success"])



import requests
from requests.models import Response
from rest_framework.test import APIClient
from jaseci.utils.utils import TestCaseHelper
from django.test import TestCase
from django.contrib.sites.models import Site
from allauth.socialaccount.models import SocialApp
from allauth.socialaccount.providers.oauth2.client import OAuth2Client
from jaseci_serv.base.models import User

old_get = requests.get


def mocked_get_access_token(*ignored):
    return {
        "access_token": "ya29.a0AX9GBdVvL b7JjkGJmFOy_oBhQC4jUdOKOjvpHGrcyFjCvWeqwW_1yTkWHJxA6jqEfXYkSzvwBCbq6XsA0o-fTDybsR9T8v8ilXM43IvqIQavWqxebTdjQo5ikPXDabrpJWQ3Uf1gMv1SfnwfeH6KdXxkGwPicaCgYKARwSARMSFQHUCsbCQepIVdu7SMuLZ5c-iI8Awg0163",
        "expires_in": 3599,
        "refresh_token": "1//0evz2w3jKj1qUCgYIARAAGA4SNwF-L9IrCiih-u1fKn9ECfKNMtKTK-y6wNZWcTfN8FO1oLnws4QqHkZZn2GSC-GUirdnOORUDIc",
        "scope": "openid https://www.googleapis.com/auth/userinfo.email https://www.googleapis.com/auth/userinfo.profile",
        "token_type": "Bearer",
        "id_token": "eyJhbGciOiJSUzI1NiIsImtpZCI6IjAyYTYxZWZkMmE0NGZjMjE1MTQ4ZDRlZmZjMzRkNmE3YjJhYzI2ZjAiLCJ0eXAiOiJKV1QifQ.eyJpc3MiOiJhY2NvdW50cy5nb29nbGUuY29tIiwiYXpwIjoiMzg0NzUyNjExMTkwLWUwNGpiNDNpOGNuZnA5NTNyMW0xZjVtcTVhYmQ5MTlpLmFwcHMuZ29vZ2xldXNlcmNvbnRlbnQuY29tIiwiYXVkIjoiMzg0NzUyNjExMTkwLWUwNGpiNDNpOGNuZnA5NTNyMW0xZjVtcTVhYmQ5MTlpLmFwcHMuZ29vZ2xldXNlcmNvbnRlbnQuY29tIiwic3ViIjoiMTE2NjIwMzI4OTM4NjYyMjkwNDI3IiwiZW1haWwiOiJqYXNlY2kuZGV2QGdtYWlsLmNvbSIsImVtYWlsX3ZlcmlmaWVkIjp0cnVlLCJhdF9oYXNoIjoiWUd5d2tPZG1VVEo5dU1ma3ZxRGNPZyIsImlhdCI6MTY3MTcwMTQ0MiwiZXhwIjoxNjcxNzA1MDQyfQ.jR23tvfhJYjxHVzOZsdNphZ_2vJ4iPIraY_cbwM9NerfmdiBEhGxhhmksF8X2K5G4Yg607StTjhKNE0XStiWhhBzZu3w7ArujqPoL2NgT2AR8kyQ91cGZcl6REKwhZBQHc7oX9V-UmK6wo8bWfPgdZ0xq_D_YDFaJuabph9d7YutGy4dsEjMkgWN-E1IdVXlXPBciaBK6Djx06x4Wr_dlfeZl0KWHcxWTDXS1DV3rEFpyZlqSTYsTVdnP-nRJZqXbgN7IRNgJIehxBleNRoXCvmm5rD5D1NCI84OOplUEYlpH_dNF8iSdrh7GLgijKl6kbKGlbWyrrBYeJa07nnvJw",
    }


def mocked_get(*ignored_args, **ignored_kwargs):
    if ignored_args[0] == "https://www.googleapis.com/oauth2/v1/userinfo":
        mocked_response = Response()
        mocked_response.status_code = 200
        mocked_response._content = b'{"id":"116620328938662290427","email":"jaseci.dev@gmail.com","verified_email":true,"name":"jaseci dev","given_name":"jaseci","family_name":"dev","picture":"https://lh3.googleusercontent.com/a/AEdFTp5FRawOuoF9Tl5jYkExTn1kGbDXqNoJP9ejF6No=s96-c","locale":"en-US"}'
        return mocked_response
    else:
        return old_get(*ignored_args, **ignored_kwargs)


class SocialAuthTest(TestCaseHelper, TestCase):
    """Test the Social Authentication"""

    def setUp(self):
        super().setUp()
        self.client = APIClient()
        OAuth2Client.get_access_token = mocked_get_access_token
        requests.get = mocked_get
        self.google_app = SocialApp.objects.create(
            provider="google",
            name="Google OAuth",
            client_id="384752611190-e04jb43i8cnfp953r1m1f5mq5abd919i.apps.googleusercontent.com",
            secret="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",
        )
        self.google_app.sites.add(
            Site.objects.first(),
            Site.objects.create(domain="google.com", name="google"),
        )
        self.google_app.save()

    def tearDown(self):
        super().tearDown()

    def test_social_auth_flow(self):
        """Test social auth flow from 'with autorization code'"""

        # no user yet
        self.assertEqual(0, User.objects.count())

        res = self.client.post(
            path="/auth/google/",
            data={
                "id_token": "",
                "code": "4%2F0AWgavdcDFN1oZoRrqBlirNICcBprusdJpC0LPYnARjbWOgHGsxJAiw0hVD9zQL3y-rPwnA",
                "access_token": "",
            },
        ).data

        # user should be created
        self.assertEqual(1, User.objects.count())
        created_user = User.objects.first()

        self.client.credentials(HTTP_AUTHORIZATION="token " + res["token"])

        # token should work
        res = self.client.get(path="/user/manage/").data

        self.assertEqual(
            res,
            {
                "id": 1,
                "email": "jaseci.dev@gmail.com",
                "name": "jaseci dev",
                "is_activated": True,
                "is_superuser": False,
            },
        )

        self.assertEqual(created_user.id, res["id"])
        self.assertEqual(created_user.email, res["email"])
        self.assertEqual(created_user.name, res["name"])
        self.assertEqual(created_user.is_activated, res["is_activated"])
        self.assertIsNotNone(created_user.get_master())

        # getting token again should create another user
        res = self.client.post(
            path="/auth/google/",
            data={
                "id_token": "",
                "code": "4%2F0AWgavdcDFN1oZoRrqBlirNICcBprusdJpC0LPYnARjbWOgHGsxJAiw0hVD9zQL3y-rPwnA",
                "access_token": "",
            },
        ).data

        # user count should still be the same
        self.assertEqual(1, User.objects.count())
        self.assertEqual(created_user, User.objects.first())



from pyexpat import model
from jaseci_serv.base.models import *
from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _


class SocialLoginProvider(models.TextChoices):
    GOOGLE = "GOOGLE", _("Google")
    FACEBOOK = "FACEBOOK", _("Facebook")
    MICROSOFT = "MICROSOFT", _("Microsoft")
    GITHUB = "GITHUB", _("GitHub")
    OKTA = "OKTA", _("Okta")
    OPENID = "OPENID", _("OpenID")


PROVIDERS_MAPPING = {
    SocialLoginProvider.GOOGLE: {
        "URL_KEY": SocialLoginProvider.GOOGLE + "_REDIRECT_URI",
        "LOGIN_URL": "https://accounts.google.com/o/oauth2/v2/auth?redirect_uri={callback_url}&prompt=consent&response_type=code&client_id={client_id}&scope=openid%20email%20profile&access_type=offline",
        "DEFAULT_REDIRECT_URI": "/auth/examples/google/",
    },
    SocialLoginProvider.FACEBOOK: {
        "URL_KEY": SocialLoginProvider.FACEBOOK + "_REDIRECT_URI",
        "LOGIN_URL": "https://www.facebook.com/v15.0/dialog/oauth?client_id={client_id}&redirect_uri={callback_url}&state={{st=state123abc,ds=123456789}}",
        "DEFAULT_REDIRECT_URI": "/auth/examples/facebook/",
    },
    SocialLoginProvider.GITHUB: {
        "URL_KEY": SocialLoginProvider.GITHUB + "_REDIRECT_URI",
        "LOGIN_URL": "",
        "DEFAULT_REDIRECT_URI": "/auth/examples/github/",
    },
    SocialLoginProvider.MICROSOFT: {
        "URL_KEY": SocialLoginProvider.MICROSOFT + "_REDIRECT_URI",
        "LOGIN_URL": "",
        "DEFAULT_REDIRECT_URI": "/auth/examples/microsoft/",
    },
    SocialLoginProvider.OKTA: {
        "URL_KEY": SocialLoginProvider.OKTA + "_REDIRECT_URI",
        "LOGIN_URL": "",
        "DEFAULT_REDIRECT_URI": "/auth/examples/okta/",
    },
    SocialLoginProvider.OPENID: {
        "URL_KEY": SocialLoginProvider.OPENID + "_REDIRECT_URI",
        "LOGIN_URL": "",
        "DEFAULT_REDIRECT_URI": "/auth/examples/openid/",
    },
}


class SingletonModel(models.Model):
    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        self.pk = 1
        super(SingletonModel, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        pass

    @classmethod
    def load(cls):
        obj, created = cls.objects.get_or_create(pk=1)
        return obj



"""Built in actions for Jaseci"""
import json
import stripe
from datetime import datetime
from jaseci.actions.live_actions import jaseci_action


def set_api_key(meta):
    api_key = meta["h"].get_glob("STRIPE_API_KEY")

    if not api_key:
        raise Exception(
            {
                "message": "Stripe is not yet configured. Please set a valid stripe key.",
                "success": False,
            }
        )

    stripe.api_key = api_key


@jaseci_action()
def create_product(
    name: str, description: str, mock_api: bool = False, meta: dict = {}
):
    """create product"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.Product.create(name=name, description=description)
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def create_product_price(
    productId: str,
    amount: int,
    currency: str,
    interval: str,
    mock_api: bool = False,
    meta: dict = {},
):
    """modify product price"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.Price.create(
            product=productId,
            unit_amount=amount,
            currency=currency,
            recurring={"interval": interval},
        )
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def product_list(detailed: bool, mock_api: bool = False, meta: dict = {}):
    """retrieve all producs"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    if not detailed:
        try:
            return stripe.Product.list(active=True)
        except Exception as e:
            return {"message": str(e)}
    else:  # retrieve product price
        try:
            return stripe.Product.list()
        except Exception as e:
            return {"message": str(e)}


@jaseci_action()
def create_customer(
    email: str,
    name: str,
    metadata: dict or None = None,
    address: dict or None = None,
    payment_method_id: str or None = None,
    mock_api: bool = False,
    meta: dict = {},
):
    """create customer"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.Customer.create(
            email=email,
            name=name,
            metadata=metadata,
            address=address,
            payment_method=payment_method_id,
            invoice_settings={"default_payment_method": payment_method_id},
        )
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def get_customer(customer_id: str, mock_api: bool = False, meta: dict = {}):
    """retrieve customer information"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.Customer.retrieve(customer_id)
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def attach_payment_method(
    payment_method_id: str, customer_id: str, mock_api: bool = False, meta: dict = {}
):
    """attach payment method to customer"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        paymentMethods = get_payment_methods(customer_id, meta)

        paymentMethod = stripe.PaymentMethod.attach(
            payment_method_id, customer=customer_id
        )

        if len(paymentMethods.data) == 0:
            update_default_payment_method(customer_id, payment_method_id, meta)

        paymentMethod.is_default = len(paymentMethods.data) == 0

        return paymentMethod

    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def delete_payment_method(
    payment_method_id: str, mock_api: bool = False, meta: dict = {}
):
    """detach payment method from customer"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.PaymentMethod.detach(payment_method_id)
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def get_payment_methods(customer_id: str, mock_api: bool = False, meta: dict = {}):
    """get customer list of payment methods"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.PaymentMethod.list(
            customer=customer_id,
            type="card",
        )
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def update_default_payment_method(
    customer_id: str, payment_method_id: str, mock_api: bool = False, meta: dict = {}
):
    """update default payment method of customer"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.Customer.modify(
            customer_id,
            invoice_settings={"default_payment_method": payment_method_id},
        )
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def create_invoice(customer_id: str, mock_api: bool = False, meta: dict = {}):
    """create customer invoice"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.Invoice.create(customer=customer_id)
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def get_invoice_list(
    customer_id: str,
    subscription_id: str,
    starting_after: str = "",
    limit: int = 10,
    mock_api: bool = False,
    meta: dict = {},
):
    """retrieve customer list of invoices"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        if starting_after != "":
            invoices = stripe.Invoice.list(
                customer=customer_id,
                limit=limit,
                starting_after=starting_after,
                subscription=subscription_id,
            )
        else:
            invoices = stripe.Invoice.list(
                customer=customer_id, limit=limit, subscription=subscription_id
            )
        return invoices
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def get_payment_intents(
    customer_id: str,
    starting_after: str = "",
    limit: int = 10,
    mock_api: bool = False,
    meta: dict = {},
):
    """get customer payment intents"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        if starting_after != "":
            payment_intents = stripe.PaymentIntent.list(
                customer=customer_id,
                limit=limit,
                starting_after=starting_after,
            )
        else:
            payment_intents = stripe.PaymentIntent.list(
                customer=customer_id, limit=limit
            )
        return payment_intents
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def create_payment_intents(
    customer_id: str,
    amount: int,
    currency: str,
    payment_method_types: str = "card",
    mock_api: bool = False,
    meta: dict = {},
):
    """Create customer payment"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.PaymentIntent.create(
            customer=customer_id,
            amount=amount,
            currency=currency,
            payment_method_types=[payment_method_types],
        )
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def get_customer_subscription(
    customer_id: str, mock_api: bool = False, meta: dict = {}
):
    """retrieve customer subcription list"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        subscription = stripe.Subscription.list(customer=customer_id)

        if len(subscription.data) == 0:
            return {"status": "inactive", "message": "Customer has no subscription"}

        return subscription
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def create_payment_method(
    card_type: str, card: dict, mock_api: bool = False, meta: dict = {}
):
    """create payment method"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.PaymentMethod.create(type=card_type, card=card)

    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def create_trial_subscription(
    payment_method_id: str,
    price_id: str,
    customer_id: str,
    trial_period_days: int = 14,
    mock_api: bool = False,
    meta: dict = {},
):
    """create customer trial subscription"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        # attach payment method to customer
        attach_payment_method(payment_method_id, customer_id, meta)

        # set card to default payment method
        update_default_payment_method(customer_id, payment_method_id, meta)

        return stripe.Subscription.create(
            customer=customer_id,
            items=[
                {"price": price_id},
            ],
            trial_period_days=trial_period_days,
        )
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def create_subscription(
    payment_method_id: str,
    price_id: str,
    customer_id: str,
    mock_api: bool = False,
    meta: dict = {},
):
    """create customer subscription"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        # attach payment method to customer
        attach_payment_method(payment_method_id, customer_id, meta)

        # set card to default payment method
        update_default_payment_method(customer_id, payment_method_id, meta)

        return stripe.Subscription.create(
            customer=customer_id,
            items=[
                {"price": price_id},
            ],
        )
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def cancel_subscription(subscription_id: str, mock_api: bool = False, meta: dict = {}):
    """cancel customer subscription"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.Subscription.delete(subscription_id)
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def get_subscription(subscription_id: str, mock_api: bool = False, meta: dict = {}):
    """retrieve customer subcription details"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.Subscription.retrieve(subscription_id)
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def update_subscription(
    subscription_id: str,
    subscription_item_id: str,
    price_id: str,
    mock_api: bool = False,
    meta: dict = {},
):
    """update subcription details"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.Subscription.modify(
            subscription_id,
            cancel_at_period_end=False,
            items=[
                {
                    "id": subscription_item_id,
                    "price": price_id,
                },
            ],
        )
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def get_invoice(invoice_id: str, mock_api: bool = False, meta: dict = {}):
    """get invoice information"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.Invoice.retrieve(invoice_id)
    except Exception as e:
        return {"message": str(e)}


@jaseci_action()
def create_usage_report(
    subscription_item_id: str, quantity: int, mock_api: bool = False, meta: dict = {}
):
    """Create usage record"""
    set_api_key(meta)

    if mock_api:
        return {"success": True}

    try:
        return stripe.SubscriptionItem.create_usage_record(
            subscription_item_id, quantity=quantity, timestamp=datetime.now()
        )
    except Exception as e:
        return {"message": str(e)}



import os

OAUTH_APPS = [
    "rest_framework.authtoken",
    "dj_rest_auth",
    "django.contrib.sites",
    "allauth",
    "allauth.account",
    "dj_rest_auth.registration",
    "allauth.socialaccount",
    "allauth.socialaccount.providers.google",
    "allauth.socialaccount.providers.facebook",
    "allauth.socialaccount.providers.github",
    "allauth.socialaccount.providers.microsoft",
    "allauth.socialaccount.providers.okta",
    # "allauth.socialaccount.providers.openid",
]


AUTH_PROVIDERS = {"facebook": "facebook", "google": "google", "email": "email"}
SOCIALACCOUNT_EMAIL_VERIFICATION = "none"

KNOX_TOKEN_EXPIRY = 24

REST_AUTH_TOKEN_MODEL = None

ACCOUNT_USER_MODEL_USERNAME_FIELD = None
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_USERNAME_REQUIRED = False
ACCOUNT_AUTHENTICATION_METHOD = "email"

SITE_ID = 1
DEFAULT_CALLBACK_URL_FOR_SSO = "/auth/examples/google/"



from django.urls import path, re_path, include
from jaseci_serv.jsx_oauth.views import *


urlpatterns = [
    path(
        "auth/",
        include(
            [
                path("get-test-url/", ExampleUrlView.as_view(), name="get_test_url"),
                path("google/", GoogleLogin.as_view(), name="google_login"),
                path("facebook/", FaceBookLogin.as_view(), name="facebook_login"),
                path("github/", GitHubLogin.as_view(), name="github_login"),
                path("microsoft/", MicrosoftLogin.as_view(), name="microsoft_login"),
                path("okta/", OktaLogin.as_view(), name="okta_login"),
                # path("openid/", OpenIdLogin.as_view(), name="openid_login"), #
                path(
                    "examples/<str:provider>/",
                    ExampleLiveView.as_view(),
                    name="oauth_example",
                ),
            ]
        ),
    ),
    re_path(r"^accounts/", include("allauth.urls"), name="socialaccount_signup"),
]



from django.contrib.auth import get_user_model
from requests.exceptions import HTTPError
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from dj_rest_auth.registration.serializers import SocialLoginSerializer

from rest_framework import serializers
from jaseci_serv.jsx_oauth.models import SocialLoginProvider

try:
    from allauth.account import app_settings as allauth_settings
    from allauth.socialaccount.helpers import complete_social_login
except ImportError:
    raise ImportError("allauth needs to be added to INSTALLED_APPS.")


class JXSSocialLoginSerializer(SocialLoginSerializer):
    def validate(self, attrs):
        view = self.context.get("view")
        request = self._get_request()

        if not view:
            raise serializers.ValidationError(
                _("View is not defined, pass it as a context variable"),
            )

        adapter_class = getattr(view, "adapter_class", None)
        if not adapter_class:
            raise serializers.ValidationError(_("Define adapter_class in view"))

        adapter = adapter_class(request)
        app = adapter.get_provider().get_app(request)

        access_token = attrs.get("access_token")
        code = attrs.get("code")
        # Case 1: We received the access_token
        if access_token:
            tokens_to_parse = {"access_token": access_token}
            token = access_token
            # For sign in with apple
            id_token = attrs.get("id_token")
            if id_token:
                tokens_to_parse["id_token"] = id_token

        # Case 2: We received the authorization code
        elif code:
            self.set_callback_url(view=view, adapter_class=adapter_class)
            self.client_class = getattr(view, "client_class", None)

            if not self.client_class:
                raise serializers.ValidationError(
                    _("Define client_class in view"),
                )

            provider = adapter.get_provider()
            scope = provider.get_scope(request)
            client = self.client_class(
                request,
                app.client_id,
                app.secret,
                adapter.access_token_method,
                adapter.access_token_url,
                self.callback_url,
                scope,
                scope_delimiter=adapter.scope_delimiter,
                headers=adapter.headers,
                basic_auth=adapter.basic_auth,
            )
            token = client.get_access_token(code)
            access_token = token["access_token"]
            tokens_to_parse = {"access_token": access_token}

            # If available we add additional data to the dictionary
            for key in ["refresh_token", "id_token", adapter.expires_in_key]:
                if key in token:
                    tokens_to_parse[key] = token[key]
        else:
            raise serializers.ValidationError(
                _("Incorrect input. access_token or code is required."),
            )
        social_token = adapter.parse_token(tokens_to_parse)
        social_token.app = app

        try:
            login = self.get_social_login(adapter, app, social_token, token)
            complete_social_login(request, login)
        except HTTPError:
            raise serializers.ValidationError(_("Incorrect value"))

        if not login.is_existing:

            if allauth_settings.UNIQUE_EMAIL:
                # check we have an account already with this email address?
                account_exists = (
                    get_user_model()
                    .objects.filter(
                        email=login.user.email,
                    )
                    .exists()
                )
                if account_exists:
                    raise serializers.ValidationError(
                        _("User is already registered with this e-mail address."),
                    )

            login.lookup()
            login.save(request, connect=True)

        attrs["user"] = login.account.user

        return attrs


class GetExampleUrlSerializer(serializers.Serializer):
    provider = serializers.ChoiceField(choices=SocialLoginProvider.choices)
